/* dosfd_img.h version 2.0.0 */
#ifndef __DOSFD_IMG_H__
#define __DOSFD_IMG_H__

#define FDI_OFFSET						(0x1000)

#define DOSFD_FILE_ENTRY_SIZE			(0x20)
#define DOSFD_FILE_ENTRY_OFFSET_ATTR	(0x0B)
#define DOSFD_FILE_ENTRY_OFFSET_TIME	(0x16)
#define DOSFD_FILE_ENTRY_OFFSET_DATE	(0x18)
#define DOSFD_FILE_ENTRY_OFFSET_CLUSTER	(0x1A)
#define DOSFD_FILE_ENTRY_OFFSET_SIZE	(0x1C)

#define DOSFD_FILE_ATTR_MASK_READ_ONLY	(0x01)
#define DOSFD_FILE_ATTR_MASK_HIDDEN		(0x02)
#define DOSFD_FILE_ATTR_MASK_SYSTEM		(0x04)
#define DOSFD_FILE_ATTR_MASK_VOL_LABEL	(0x08)
#define DOSFD_FILE_ATTR_MASK_SUBDIR		(0x10)
#define DOSFD_FILE_ATTR_MASK_ARCHIVE	(0x20)

#define DOSFD_FILE_TIME_MASK_HOUR		(0xF800)	/* 0-23 */
#define DOSFD_FILE_TIME_SHIFT_HOUR		(11)
#define DOSFD_FILE_TIME_MASK_MIN		(0x07E0)	/* 0-59 */
#define DOSFD_FILE_TIME_SHIFT_MIN		(5)
#define DOSFD_FILE_TIME_MASK_SEC2		(0x001F)	/* 0-29 * 2 */

#define DOSFD_FILE_DATE_MASK_YEAR		(0xFE00)	/* 0-127 + 1980 */
#define DOSFD_FILE_DATE_SHIFT_YEAR		(9)
#define DOSFD_FILE_DATE_MASK_MONTH		(0x01E0)	/* 1-12 */
#define DOSFD_FILE_DATE_SHIFT_MONTH		(5)
#define DOSFD_FILE_DATE_MASK_DAY		(0x001F)	/* 1-31 */

#define DOSFD_2HD_TRACKS				(77)
#define DOSFD_2HD_HEADS					(2)
#define DOSFD_2HD_SECTORS_PER_TRACK		(8)
#define DOSFD_2HD_SECTOR_SIZE			(1024)
#define DOSFD_2HD_SECTORS				(DOSFD_2HD_TRACKS * DOSFD_2HD_HEADS * DOSFD_2HD_SECTORS_PER_TRACK)	/* 1232 */
#define DOSFD_2HD_SIZE					(DOSFD_2HD_SECTORS * DOSFD_2HD_SECTOR_SIZE)
#define DOSFD_2HD_FAT_SECTOR			(1)
#define DOSFD_2HD_SECTORS_PER_FAT		(2)
#define DOSFD_2HD_ROOT_DIR_SECTOR		(5)
#define DOSFD_2HD_ROOT_DIR_ENTRIES		(192)
#define DOSFD_2HD_SECTORS_PER_ROOT_DIR	(DOSFD_2HD_ROOT_DIR_ENTRIES * DOSFD_FILE_ENTRY_SIZE / DOSFD_2HD_SECTOR_SIZE)
#define DOSFD_2HD_CLUSTER_2_SECTOR		(11)
#define DOSFD_2HD_SECTORS_PER_CLUSTER	(1)

#define DOSFD_2DD8_TRACKS				(80)
#define DOSFD_2DD8_HEADS				(2)
#define DOSFD_2DD8_SECTORS_PER_TRACK	(8)
#define DOSFD_2DD8_SECTOR_SIZE			(512)
#define DOSFD_2DD8_SECTORS				(DOSFD_2DD8_TRACKS * DOSFD_2DD8_HEADS * DOSFD_2DD8_SECTORS_PER_TRACK)	/* 1280 */
#define DOSFD_2DD8_SIZE					(DOSFD_2DD8_SECTORS * DOSFD_2DD8_SECTOR_SIZE)
#define DOSFD_2DD8_FAT_SECTOR			(1)
#define DOSFD_2DD8_SECTORS_PER_FAT		(2)
#define DOSFD_2DD8_ROOT_DIR_SECTOR		(5)
#define DOSFD_2DD8_ROOT_DIR_ENTRIES		(112)
#define DOSFD_2DD8_SECTORS_PER_ROOT_DIR	(DOSFD_2DD8_ROOT_DIR_ENTRIES * DOSFD_FILE_ENTRY_SIZE / DOSFD_2DD8_SECTOR_SIZE)
#define DOSFD_2DD8_CLUSTER_2_SECTOR		(12)
#define DOSFD_2DD8_SECTORS_PER_CLUSTER	(2)

#define DOSFD_2DD9_TRACKS				(80)
#define DOSFD_2DD9_HEADS				(2)
#define DOSFD_2DD9_SECTORS_PER_TRACK	(9)
#define DOSFD_2DD9_SECTOR_SIZE			(512)
#define DOSFD_2DD9_SECTORS				(DOSFD_2DD9_TRACKS * DOSFD_2DD9_HEADS * DOSFD_2DD9_SECTORS_PER_TRACK)	/* 1440 */
#define DOSFD_2DD9_SIZE					(DOSFD_2DD9_SECTORS * DOSFD_2DD9_SECTOR_SIZE)
#define DOSFD_2DD9_FAT_SECTOR			(1)
#define DOSFD_2DD9_SECTORS_PER_FAT		(3)
#define DOSFD_2DD9_ROOT_DIR_SECTOR		(7)
#define DOSFD_2DD9_ROOT_DIR_ENTRIES		(112)
#define DOSFD_2DD9_SECTORS_PER_ROOT_DIR	(DOSFD_2DD9_ROOT_DIR_ENTRIES * DOSFD_FILE_ENTRY_SIZE / DOSFD_2DD9_SECTOR_SIZE)
#define DOSFD_2DD9_CLUSTER_2_SECTOR		(14)
#define DOSFD_2DD9_SECTORS_PER_CLUSTER	(2)

#define DOSFD_MIN_CLUSTER				(0x002)
#define DOSFD_MAX_CLUSTER				(0xFEF)


struct dosfd_image_t {
	int img_offset;
	int disk_size;
	unsigned char *disk_img;
	int fat_size;
	int *fat;
	int num_tracks;
	int num_heads;
	int sectors_per_track;
	int sector_size;
	int num_sectors;
	int fat_sector;
	int sectors_per_fat;
	int root_dir_sector;
	int root_dir_entries;
	int sectors_per_root_dir;
	int cluster_2_sector;
	int sectors_per_cluster;
};

extern void free_dosfd_image(struct dosfd_image_t *image, int free_all);
extern int read_dosfd_image(struct dosfd_image_t *image, const char *image_name, int img_offset);
extern int get_dosfd_image_file_size(const struct dosfd_image_t *image, const char *file_name, int ignore_case);
extern int read_dosfd_image_file(const struct dosfd_image_t *image, const char *file_name, unsigned char *data, int data_size, int ignore_case);
extern int write_dosfd_image(const struct dosfd_image_t *image, const char *image_name);
extern int write_dosfd_image_file(struct dosfd_image_t *image, const char *file_name, const unsigned char *data, int data_size, int ignore_case);

#endif	/* __DOSFD_IMG_H__ */
