#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define PLUTO_PCAT_MAP_FILE		"MAPG80"
#define PLUTO_PC98_MAP_FILE		"u2pc98pluto.map"

#define MAP_XSIZE	(64)
#define MAP_YSIZE	(64)
#define MAP_SIZE	(MAP_XSIZE * MAP_YSIZE)

static unsigned char map[MAP_SIZE];
static unsigned char map98[MAP_SIZE];

static const unsigned char conv_table[64] = {
	0x00, 0x01, 0x02, 0x03, 0x04, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x34, 0x36, 0x38, 0x3A, 0x3C,
	0x3E, 0x2E, 0x2F, 0x30, 0x31, 0x2D, 0x2C, 0x05, 0x40, 0x42, 0x44, 0x46, 0x06, 0x0D, 0x0E, 0x0F,
	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
	0x2B, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x4A, 0x4C, 0x4E, 0x50
};

static const unsigned char water_table[16] = {
	0x00, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72
};

#define IS_WATER(_m)	((_m) == 0x00 || (_m) == (0x0B << 2) || (_m) == (0x12 << 2))

int main(int argc, char **argv)
{
	int result = 1;
	FILE *fp = NULL;
	int x;
	int y;
	int i;

	if ((fp = fopen(PLUTO_PCAT_MAP_FILE, "rb")) == NULL) {
		printf("ERROR: open error: %s\n", PLUTO_PCAT_MAP_FILE);
		goto ERROR_EXIT;
	}

	if (fread(map, sizeof(map), 1, fp) != 1) {
		printf("ERROR: read error: %s\n", PLUTO_PCAT_MAP_FILE);
		goto ERROR_EXIT;
	}

	fclose(fp);
	fp = NULL;

	i = 0;
	for (y = 0; y < MAP_YSIZE; y++) {
		for (x = 0; x < MAP_XSIZE; x++, i++) {
			map98[i] = conv_table[map[i] >> 2];
			if (0x2E <= map98[i] && map98[i] < 0x64) {
				printf("(%02X, %02X) = %02X\n", x, y, map98[i]);

				if (map98[i] == 0x2F || map98[i] == 0x34) {
					map98[i] = 0x00;
				} else {
					map98[i] = 0x02;
				}
			}
			if (IS_WATER(map[i])) {
				int offset = 0;
				int j;

				j = i + MAP_XSIZE;
				if (MAP_SIZE <= j) {
					j -= MAP_SIZE;
				}
				if (! IS_WATER(map[j])) {
					offset += 1;
				}

				j = i - MAP_XSIZE;
				if (j < 0) {
					j += MAP_SIZE;
				}
				if (! IS_WATER(map[j])) {
					offset += 2;
				}

				j = i + 1;
				if (MAP_XSIZE <= x + 1) {
					j -= MAP_XSIZE;
				}
				if (! IS_WATER(map[j])) {
					offset += 4;
				}

				j = i - 1;
				if (x - 1 < 0) {
					j += MAP_XSIZE;
				}
				if (! IS_WATER(map[j])) {
					offset += 8;
				}

				map98[i] = water_table[offset];
			}
		}
	}

	if ((fp = fopen(PLUTO_PC98_MAP_FILE, "wb")) == NULL) {
		printf("ERROR: open error: %s\n", PLUTO_PC98_MAP_FILE);
		goto ERROR_EXIT;
	}

	if (fwrite(map98, sizeof(map98), 1, fp) != 1) {
		printf("ERROR: write error: %s\n", PLUTO_PC98_MAP_FILE);
		goto ERROR_EXIT;
	}

	fclose(fp);
	fp = NULL;

	result = 0;

ERROR_EXIT:
	if (fp != NULL) {
		fclose(fp);
	}

	return result;
}
