#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>


#define U2_SAVE_FILE_SIZE	(823296)

static const int save_offset[4] = {
	0x015800,
	0x022000,
	0x02E800,
	0x03B000
};

static const unsigned char mark[6] = {
	0x4A, 0x48, 0x31, 0x4E, 0x52, 0x52
};

static const unsigned char update_data[8 * 4] = {
	0x09, 0x28, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00,
	0x32, 0x2E, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00,
	0x0F, 0x2F, 0x00, 0x00, 0x2F, 0x00, 0x00, 0x00,
	0x3D, 0x3D, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00
};

static unsigned char save_data[U2_SAVE_FILE_SIZE];


static void usage(void);
static char *create_write_name(const char *name);
static int read_save_file(const char *filename);
static int write_save_file(const char *filename, int overwrite);
static int update_save_data(int save_no);
static int confirm(const char *msg);


int main(int argc, char **argv)
{
	int result = 1;
	int num_cmd_params = 1;
	const char *img_file = "U2_Save.fdi";
	const char *out_file = NULL;
	char *out_file_buf = NULL;

	int save_no = 0;
	const int save_no_mask[4] = { 1, 2, 4, 8 };

	int opt_overwrite = 0;

	while (*++argv != NULL) {
		if (**argv == '-') {
			if (strcmp(*argv, "-?") == 0 || strcmp(*argv, "--help") == 0) {
				usage();
				return 1;
			} else if (strcmp(*argv, "-1") == 0) {
				save_no |= save_no_mask[0];
			} else if (strcmp(*argv, "-2") == 0) {
				save_no |= save_no_mask[1];
			} else if (strcmp(*argv, "-3") == 0) {
				save_no |= save_no_mask[2];
			} else if (strcmp(*argv, "-4") == 0) {
				save_no |= save_no_mask[3];
			} else if (strcmp(*argv, "-a") == 0 || strcmp(*argv, "--all") == 0) {
				save_no = 0xF;
			} else if (strcmp(*argv, "-o") == 0 || strcmp(*argv, "--write") == 0) {
				if (*++argv == NULL) {
					printf("ERROR: no output filename\n");
					return 1;
				}
				out_file = *argv;
#if 0
			} else if (strcmp(*argv, "--overwrite") == 0) {
				opt_overwrite = 1;
#endif
			} else {
				printf("unknown option '%s'\n", *argv);
				return 1;
			}
		} else {
			if (num_cmd_params <= 0) {
				printf("illegal parameter '%s'\n", *argv);
				return 1;
			}
			img_file = *argv;
			num_cmd_params--;
		}
	}

	if (save_no == 0) {
		printf("ERROR: save no not specified\n");
		return 1;
	}

	if (out_file == NULL) {
		if ((out_file_buf = create_write_name(img_file)) == NULL) {
			printf("ERROR: can't create filename\n");
			goto ERROR_EXIT;
		}
		out_file = out_file_buf;
	}

	if (read_save_file(img_file) < 0) {
		printf("ERROR: can't read save file '%s'\n", img_file);
		goto ERROR_EXIT;
	}

	{
		int i;
		for (i = 0; i < 4; i++) {
			if ((save_no & save_no_mask[i]) != 0 && update_save_data(i) < 0) {
				printf("ERROR: can't update save data '%s'\n", img_file);
				goto ERROR_EXIT;
			}
		}
	}

	if (write_save_file(out_file, opt_overwrite) < 0) {
		printf("ERROR: can't write save file '%s'\n", out_file);
		goto ERROR_EXIT;
	}

	result = 0;

ERROR_EXIT:
	if (out_file_buf != NULL) {
		free(out_file_buf);
	}

	return result;
}

static void usage(void)
{
	printf("usage: u2pc98pluto_save [options] [DISK-IMG]\n");
	printf("option: -o|--write OUTPUT-IMG   specify output file name\n");
	printf("        -1|-2|-3|-4             specify save data # to update\n");
	printf("        -a|--all                update all save data\n");
#if 0
	printf("           --overwrite          overwrite output files\n");
#endif
	printf("        -?|--help               print usage\n");
	printf("\n");
	printf("read disk image from 'U2_Save.fdi' if not specified\n");
}

static char *create_write_name(const char *name)
{
	int baselen;
	const char *ext;
	int len = strlen(name) + 5;
	char *new_name = (char *)malloc(len);
	if (new_name == NULL) {
		printf("memory exhaust!\n");
		return NULL;
	}

	if ((ext = strrchr(name, '.')) == NULL) {
		baselen = strlen(name);
		ext = "";
	} else {
		baselen = ext - name;
	}

	strncpy(new_name, name, baselen);
	strcpy(new_name + baselen, ".NEW");
	strcpy(new_name + baselen + 4, ext);

	return new_name;
}

static int read_save_file(const char *filename)
{
	int result = -1;
	FILE *fp = NULL;
	int size;

	if ((fp = fopen(filename, "rb")) == NULL) {
		printf("ERROR: open error: %s\n", filename);
		goto ERROR_EXIT;
	}

	if (fseek(fp, 0, SEEK_END) < 0) {
		printf("ERROR: seek error: SEEK_END: %s\n", filename);
		goto ERROR_EXIT;
	}

	size = ftell(fp);

	if (fseek(fp, 0, SEEK_SET) < 0) {
		printf("ERROR: seek error: SEEK_SET: %s\n", filename);
		goto ERROR_EXIT;
	}

	if (size != U2_SAVE_FILE_SIZE) {
		printf("ERROR: file size mismatch: %s\n", filename);
		goto ERROR_EXIT;
	}

	if (fread(save_data, sizeof(save_data), 1, fp) != 1) {
		printf("ERROR: read error: %s\n", filename);
		goto ERROR_EXIT;
	}

	fclose(fp);
	fp = NULL;

	result = 0;

ERROR_EXIT:
	if (fp != NULL) {
		fclose(fp);
	}

	return result;
}

static int write_save_file(const char *filename, int overwrite)
{
	int result = -1;
	FILE *fp = NULL;
	struct stat st;

	if (overwrite == 0 && stat(filename, &st) == 0) {
		printf("already exists '%s'\n", filename);
		if (confirm("overwrite ?") <= 0) {
			goto ERROR_EXIT;
		}
	}

	if ((fp = fopen(filename, "wb")) == NULL) {
		printf("ERROR: open error: %s\n", filename);
		goto ERROR_EXIT;
	}

	if (fwrite(save_data, sizeof(save_data), 1, fp) != 1) {
		printf("ERROR: write error: %s\n", filename);
		goto ERROR_EXIT;
	}

	fclose(fp);
	fp = NULL;

	result = 0;

ERROR_EXIT:
	if (fp != NULL) {
		fclose(fp);
	}

	return result;
}

static int update_save_data(int save_no)
{
	unsigned char *ptr = save_data + save_offset[save_no];
	int i;

	if (memcmp(ptr, mark, sizeof(mark)) != 0) {
		printf("ERROR: save data not found: #%d\n", save_no + 1);
		return -1;
	}

	ptr += 16;
	for (i = 0; i < 8 * 5; i++) {
		if (*(ptr + i) != 0) {
			printf("WARNING: data #%d is used\n", save_no + 1);
			break;
		}
	}

	ptr += 8;
	memcpy(ptr, update_data, sizeof(update_data));

	return 0;
}

static int confirm(const char *msg)
{
	printf("%s (Y or N): ", msg);
	fflush(stdout);

	while (1) {
		int c = getchar();
		if (c == 'y' || c == 'Y') {
			return 1;
		}
		if (c == 'n' || c == 'N') {
			return 0;
		}
	}

	return -1;
}

/* EOF */
